% ----------------------------------------------------------------------- %
% % this program replicates Gustman and Steinmeier (2008) C++ Model     % %
% % Ver 1.0 10.28.2013                                                  % %
% ----------------------------------------------------------------------- %


diary('GSmodel.log')
clear
clc
tic


%%%%%%%%%%%%%%%
% choose mode %
%%%%%%%%%%%%%%%
% this program could do various jobs by calling different modes;
% users could change value of mode below to indicate which type of work to do;
% test mode (0): test the codes by creating a synthetic individual whose characteristics are given in function setdata;
% iteration mode (1): do a single q function evaluation, given initial parameter values and the HRS data;
% estimation mode (2): estimate preference parametes using the HRS data;
% simulation mode (3): simulate retirement moments based on the estimated parameters (must follow mode 2);
mode = 0; % mode;


%%%%%%%%%%%%%%%%%
% choose policy %
%%%%%%%%%%%%%%%%%

% this program could examine the effect of various Social Security policies on labor supply decisions;
% users could change value of policy below to indicate which type of policy to test;
% baseline policy (0): apply the actual Social Security rules;
% counterfactual policy 1 (1): apply Social Security rules advanced 6 years (the rules of a cohort 6 years younger);
% counterfactual policy 2 (2): apply Social Security rules advanced 12 years (the rules of a cohort 12 years younger);
policy = 0; % policy;


%%%%%%%%%%%%%
% define gv %
%%%%%%%%%%%%%
% gv stands for global variable;
% it is a structure variable that carries the pre-determined, unchanged parameters;
% users could change the parameters values based on needs;

% control related variables;
gv.mode = mode; % program mode;
gv.policy = policy; % program policy;
gv.maxobs = 2500; % maximum number of observations after treatment from the HRS/to be evaluated using dynamic programs; it is for test purpose only, not involved in any calculations;
gv.nparms = 8; % number of preference parameters to be estimated; 
gv.nmoments = 43; % number of moments to be matched;
gv.nsims = 10000; % number of simulations for each household;

% grid size variables; it could be increased to get more accurate estimates;
gv.nacatlim = 40; % grid size of asset;
gv.ntcatlim = 6; % grid size of partial retirement preference;
gv.necatlim = 17; % grid size of leisure preference shock;
gv.nlcatlim = 10; % grid size of DC pension balance;
gv.npcatlim = 4; % grid size of DB pension benefits;
gv.nrcatlim = 17; % grid size of asset return shock;
gv.nscatlim = 4; % grid size of Social Security benefits;
gv.nvcatlim = 3; % grid size of survival status;
gv.ndcatlim = 2; % indicator of delayed DB pension, 1 is no, 2 is yes;

% starting and terminal age;
% everybody enters the model at the starting age with $0 in asset;
% everybody dies after the terminal age;
gv.t0 = 25; % starting age; 
gv.T = 99; % terminal age; the max age of life; 

% initial values of preference parameters;
gv.parm.alpha = -0.1600; % consumption parameter, transform of CRRA coefficient (alpha = 1 - crra);
% leisure preference related parameters; 
% h(t) = exp(beta*X(t) + epsilon(t));
% epsilon(t) = erho * epsilon(t-1) + u(sige);
% individuals start with a random drawn of epsilon from a distribution with mean 0 and standard deviation sige;
% he keeps this random preference until he retires from his career job;
% then epsilon is allowed to change with an autocorrelation coefficient erho, so individuals could come back to work after retirement; 
gv.parm.sige = 5.8700; % standard deviation of random leisure preference shock;
gv.parm.erho = 0.7100; % autocorrelation of epsilon after quitting career job;
gv.parm.beta0 = -9.8000; % beta vector: constant;
gv.parm.beta1 = 0.0670; % beta vector: coefficient of (age - 62);
gv.parm.beta2 = 5.7000; % beta vector: health;
% gamma defines the preference (utility) for partial retirement; 
% Vp = L^gamma;
% Vp = 0.5 + 0.5 * log(percentile / 100 * (exp(gamma0 + gamma1 * (age - 65)) - 1) + 1) / (gamma0 + gamma1 * (age - 65));
% percentile shows six level (gv.ntcatlim) of preference, the current six categories are 5, 20, 40, 60, 80, 95;
% if age > healthage Vp = 0.5;
gv.parm.gamma0 = -3.6200; % gamma vector: constant;
gv.parm.gamma1 = 0.1600; % gamma vector: coefficient of (age - 65);

% life table;
% lifetable shows survival probabilities by cohort, gender and age;
% it comes from COH0015.95T, COH1655.95T and COH5690.95T, column l(x);
% l(x) shows how many of the original 100,000 individuals are still alive at the specific age;
% in original GSmodel, it reads in the text files directly;
% the text files are saved into .mat to speed up Matlab program;
% the format is as follows:
% survetable(birthyear, gender, age)
% birthyear position is 1 if born in 1900, 2 if born in 1901, and so on;
% gender position is 1 if male, 2 if female;
% age position is 1 indicating age 0 survival numbers, 2 indicating age 1 survival numbers, and so on;
load survtable;
gv.survtable = survtable; % life table;

% asset related variables;
gv.inflrate = 0.040; % inflation rate, fixed at all time;
gv.penadjrate = 0.38; % fraction of inflation that is offset by DB pension increases, so the rate to adjust pension income each year is gv.inflrate * (1- gv.penadjrate);
% actual weighted real asset return (portfolio allocation is 50% stocks, 5% bonds and 45% cash and CD) from 1926 to 2000;
% data from ibbotson.xls, ibbotson valuation edition yearbook 2001;
gv.realrate = [9.14;  22.65;  24.40;  -2.09;  -5.11; ...
    -11.90;   7.40;  26.62;  -2.23;  21.15;  16.17; -20.46;  18.58;   0.55;  -5.58; ...
    -15.45;   1.15;  10.04;   8.04;  16.60; -22.04;  -6.05;   0.56;  11.99;  10.61; ...
    6.64;   9.11;  -0.12;  27.52;  16.07;   1.29;  -6.65;  20.35;   5.72;   0.59; ...
    13.79;  -4.03;  11.22;   8.82;   6.13;  -6.05;  10.45;   3.17;  -7.59;   0.03; ...
    6.39;   8.08; -13.03; -21.60;  14.64;  10.18;  -8.06;  -2.54;   0.58;   8.75; ...
    -4.62;  13.45;  11.49;   4.36;  17.22;  12.01;   0.57;   7.31;  15.72;  -3.86; ...
    15.63;   2.89;   4.39;  -0.59;  20.15;  10.54;  18.09;  15.48;   9.57;  -4.31];    
gv.realrateg = 0.0474; % geometric mean of real returns; (r1 * r2 * ... * rn) ^ (1 / n);
gv.realratea = 0.0533; % arithmatic mean of real returns; (r1 + r2 + ... + rn) / n;
gv.realratesd = 0.1105; % standard deviation of real returns; it appears too low;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% create or read in oridata from the HRS %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% oridata is a data structure variable; it contains information of all observations created / from the HRS; 
if gv.mode == 0; % test mode;
    
    % in test mode, this program will create one hypothetical household to test the codes;
    oridata = setdata(gv);
    gv.nobs = 1; % number of valid observations after treatment from the HRS/to be evaluated using dynamic programs;
    
elseif gv.mode == 1 || gv.mode == 2; % iteration or estimation mode;
    gv.nobs = 2211;
    % in iteration or estimation modes, use function input to read in HRS data from GSRetirementData.dat; ???
    % gv.data = inpt(gv);
    % because GSRetirementData.dat is not available at the current stage, we use function createdata to create a hypothetical multi-household dataset;
    oridata = createdata(gv);
    gv.nobs = size(oridata, 1); % number of valid observations after treatment from the HRS/to be evaluated using dynamic programs;
    
    % report error if observation number is higher than expected;
    if gv.nobs > gv.maxobs;
        disp ('Error: too many observations');
        return;
    end;

    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % find valid observed moments for later calculation %
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % arglist is a strctural variable captures results of calculations;
    % arglist.nmat records which moments are observed from the HRS; if the value of arglist.nmat is 1, the corresponding data moment from the HRS is observed, 0 otherwise;
    arglist.nmat = zeros(gv.nobs, gv.nmoments); % indicator matrix: whether the corresponding data moments are observed from the HRS, gv.nobs X gv.nmoments.
    
    for obs = 1 : gv.nobs; % household;

        data = oridata(obs, 1); % retrive the corresponding household's information from the original HRS dataset;
        
        % moments.ind: structure variable indicating whether the corresponding data moments are observed from the HRS; a total of 43 moments, must equal to gv.nmoments defined above;
        moments.ind.age = zeros(13, 1); % 13 moments of whether fully retired at ages 54 - 66;
        moments.ind.pr = zeros(5, 1); % 5 moments of whether partial retired at ages 55, 58, 60, 62 and 65;
        moments.ind.lowinc = zeros(5, 1); % 5 moments of whether fully retired at ages 55, 58, 60, 62 and 65 if in the lower third of lifetime income;
        moments.ind.highinc = zeros(5, 1); % 5 moments of whether fully retired at ages 55, 58, 60, 62 and 65 if in the upper third of lifetime income;
        moments.ind.health = zeros(5, 1); % 5 moments of whether fully retired at ages 55, 58, 60, 62 and 65 if in poor health;
        moments.ind.healthpr = zeros(5, 1); % 5 moments of whether partial retired at ages 55, 58, 60, 62 and 65 if in poor health;
        moments.ind.reversal = zeros(5, 1); % 5 moments of whether in reversal job at ages 55, 58, 60, 62 and 65; it is 1 if the husband is working full time at the current age, given they were fully/partially retired previously;
        
        for surv = 1 : 6; % survey;
            
            age = data.age(surv, 1); % age;
            ret = data.ret(surv, 1); % retirement status;

            if age >= 54 && age <= 66 && ret >= 1 && ret <= 5; % age between 54 and 66, and retirement status is available; 1 means fully retired, 3 means partial retired (part time work), 5 means full time work (including career job and returned full time work);
                
                moments.ind.age(age - 54 + 1, 1) = 1; % observed whether fully retired at ages 54 - 66;

                % find ritirement status at 55, 58, 60, 62, and 65;
                if age == 55;
                    k =  1;
                elseif age == 58;
                    k =  2;
                elseif age == 60;
                    k =  3;
                elseif age == 62;
                    k =  4;
                elseif age == 65;
                    k =  5;
                else
                    k = 0;
                end;
                
                if k >= 1;
                    moments.ind.pr(k, 1) = 1; % observed whether partial retired at ages 55, 58, 60, 62 and 65;
                    if data.inccat == 1;
                        moments.ind.lowinc(k, 1) = 1; % observed whether fully retired at ages 55, 58, 60, 62 and 65 if in the lower third of lifetime income;
                    end;
                    if data.inccat == 3;
                        moments.ind.highinc(k, 1) = 1; % observed whether fully retired at ages 55, 58, 60, 62 and 65 if in the upper third of lifetime income;
                    end;
                    if age >= data.healthage;
                        moments.ind.health(k, 1) = 1; % observed whether fully retired at ages 55, 58, 60, 62 and 65 if in poor health;
                    end;
                    if age >= data.healthage;
                        moments.ind.healthpr(k, 1) = 1; % observed whether partial retired at ages 55, 58, 60, 62 and 65 if in poor health;
                    end;
                end;
                
            end;
            
        end;
        
        for surv = 1 : 5;
            
            age0 = data.age(surv, 1); % age;
            ret0 = data.ret(surv, 1); % retirement status;
            age1 = data.age(surv + 1, 1); % age at the next wave;
            ret1 = data.ret(surv + 1, 1); % retirement status at the next wave;

            if age0 >= 54 && age0 <= 66 && ret0 >= 1 && ret0 <= 5 && age1 >= 54 && age1 <= 66 && ret1 >= 1 && ret1 <= 5; % observed retirement status in both consecutive waves;
                moments.ind.reversal(surv, 1) = 1; % observed whether in reversal job at survey 2 - 6; 
            end; 
            
        end;

        % put everything together and save to arglist.nmat;
        arglist.nmat(obs, 1 : 13) = moments.ind.age;
        arglist.nmat(obs, 14 : 18) = moments.ind.pr;
        arglist.nmat(obs, 19 : 23) = moments.ind.lowinc;
        arglist.nmat(obs, 24 : 28) = moments.ind.highinc;
        arglist.nmat(obs, 29 : 33) = moments.ind.health;
        arglist.nmat(obs, 34 : 38) = moments.ind.healthpr;
        arglist.nmat(obs, 39 : 43) = moments.ind.reversal;
        
    end;
    
end;

if gv.mode == 0; % test mode;
    test(gv, oridata);
elseif gv.mode == 1; % iteration mode;
    iter(gv, oridata, arglist);
elseif gv.mode == 2; % estimation mode;
    qmin(gv, oridata, arglist);
elseif gv.mode == 3; % simulation mode;
    cparm = qmin(gv, oridata, arglist);
    simu(cparm, gv, oridata, arglist);
end;

fprintf('\n');
toc
diary off

















