% ----------------------------------------------------------------------- %
% % This function takes current resources (including returns on assets) % %
% % and decides how much to consume (and how much to save or dissave)   % %
% ----------------------------------------------------------------------- %

function [margu, totu, saving, totutil] = GSModelinc(vcat, y, cv, tuv, gv, calc, arglist)

% define some parameters to simplify codes;
rho = arglist.rvec(calc.obs, 1); % rho is time discount factor; 
tp = 1 + rho; % 1 / tp is time discount rate;
consadj = 2 ^ ( 0.5 * (1 - calc.alpha)); % marginal utility adjustment factor for couples; the utility functional form for couples are: sqrt(2) * (C(t)/sqrt(2)) ^ alpha / alpha;

% initialization;
yac = zeros(gv.nacatlim, 1); % income (y) - leftover asset (a) - optimal consumption given leftover asse (c);
margu = zeros(gv.nacatlim, 1); % marginal utility next period;
totu = zeros(gv.nacatlim, 1); % total utility next period;
saving = zeros(gv.nacatlim, 1); % saving (income - consumption) at the current period (could be negative); initial asset at the current period + saving at the current period = leftover asset at the current period;
totutil = zeros(gv.nacatlim, 1); % total utility if both spouses alive;

% setup boundary;
if y < 10;
    y = 10; 
end;

if tp < 2.9999; % low time preference; not necessarily all resources are consumed each period;
    
    % initialize variables about yac vector;
    reversal = 0; % whether yac is monotonic;
    yacmin = 0; % if yac is non-monotonic, the minimum value of yac before reversal;
    yacmax = 0; % if yac is non-monotonic, the maximum value of yac after reversal;

    % decide whether yac is monotonic; if yac is non-monotonic, need to check for global maximum;
    yac(1, 1) = y - cv(1, 1); % because calc.acats(1, 1) = 0;
    for k = 2 : gv.nacatlim;
        yac(k, 1) = y - calc.acats(k, 1) - cv(k, 1); % income (y) - leftover asset (a) - optimal consumption given leftover asse (c);
        if yac(k, 1) > yac(k - 1, 1);
            if reversal == 0;
                yacmin = yac(k - 1, 1);
                yacmax = yac(k, 1);
                reversal = 1;
            else
                if yac(k - 1, 1) < yacmin;
                    yacmin = yac(k - 1, 1);
                end;
                if yac(k, 1) > yacmax;
                    yacmax = yac(k, 1);
                end;
            end;
        end;
    end;

    % find optimal consumption at the current period given initial asset;
    % interpolation;
    k = 1; % initialize leftover asset grid;
    for acat = 1 : gv.nacatlim; % asset;
        
        nassets = - calc.acats(acat, 1);

        if reversal == 0 || nassets > yacmax || nassets < yacmin;   % monotonic yac function around the evaluation point;
            
            if yac(k, 1) < nassets;
                
                k = k - 1;
                while k >= 1 && yac(k, 1) < nassets;
                    k = k - 1;
                end;
                
                if k < 1; % optimal to leave no assets to next period;
                    f = 0;
                    k = 1;
                else
                    f = (yac(k, 1) - nassets) / (yac(k, 1) - yac(k + 1, 1));
                end;
                
            else
                
                while k < gv.nacatlim && yac(k + 1, 1) >= nassets;
                    k = k + 1;
                end;
                
                if k > gv.nacatlim - 1; % requires extrapolation;
                    k = gv.nacatlim - 1;
                    if cv(k + 1, 1) > cv(k, 1);
                        f = (yac(k, 1) - nassets) / (yac(k, 1) - yac(k + 1, 1));
                    else
                        f = 1 + (yac(k + 1, 1) - nassets) / (calc.acats(k + 1, 1) - calc.acats(k, 1));
                    end;
                else
                    f = (yac(k, 1) - nassets) / (yac(k, 1) - yac(k + 1, 1));
                end;
                
            end;

            c = y + calc.acats(acat, 1) - ((1 - f) * calc.acats(k, 1) + f * calc.acats(k + 1, 1));

            if vcat == 1; % married couple adjustment;
                mu = consadj * c ^ (calc.alpha - 1); % marginal utility at the current period;
            else
                mu = c ^ (calc.alpha - 1);
            end;
            
            tu = (1 - f) * tuv(k, 1) + f * tuv(k + 1, 1) + c * mu / calc.alpha; % total utility at the current period plus total utility generated by leftover asset;
            
        else % yac nonmonotonic; check for global maximum;
            
            tumax = -9e20;

            % seperately test three types of solutions and pick the best one;
            % interpolation;
            % solution 1, regular solution;
            for k = 1 : gv.nacatlim - 1;
                if nassets <= yac(k, 1) && nassets > yac(k + 1, 1);
                    
                    f = (yac(k, 1) - nassets) / (yac(k, 1) - yac(k + 1, 1));

                    c = y + calc.acats(acat, 1) - ((1 - f) * calc.acats(k, 1) + f * calc.acats(k + 1, 1));

                    if vcat == 1; % married couple adjustment;
                        mu = consadj * c ^ (calc.alpha - 1); % marginal utility at the current period;
                    else
                        mu = c ^ (calc.alpha - 1);
                    end

                    tu = (1 - f) * tuv(k, 1) + f * tuv(k + 1, 1) + c * mu / calc.alpha;

                    if tu > tumax; % total utility is higher than the total utility from the previous solution;
                        cmax  = c; % consumption;
                        mumax = mu; % marginal utility;
                        tumax = tu; % total utility;
                    end;
                    
                end;
            end;

            % solution 2; corner solution;
            if yac(1, 1) < nassets; % potential corner solution at acats = 0 (leftover asset is 0);
                
                c = y + calc.acats(acat, 1); % consume everything;

                if vcat == 1; % married couple adjustment;
                    mu = consadj * c ^ (calc.alpha - 1); % marginal utility at the current period;
                else 
                    mu = c ^ (calc.alpha - 1);
                end;

                tu = tuv(1, 1) + c * mu / calc.alpha; % total utility at the current period; 

                if tu > tumax; % total utility is higher than the total utility from the previous solution;
                    cmax  = c; % consumption;
                    mumax = mu; % marginal utility;
                    tumax = tu; % total utility;
                end;
                
            end;

            % solution 3; corner solution;
            if yac(gv.nacatlim, 1) >= nassets; % potential corner solution at acats > acats(gv.nacatlim, 1);
                
                k = gv.nacatlim - 1;

                if cv(k + 1, 1) > cv(k, 1);
                    f = (yac(k, 1) - nassets) / (yac(k, 1) - yac(k + 1, 1));
                else
                    f = 1 + (yac(k + 1, 1) - nassets) / (calc.acats(k + 1, 1) - calc.acats(k, 1));
                end;

                c = y + calc.acats(acat, 1) - ((1 - f) * calc.acats(k, 1) + f * calc.acats(k + 1, 1)); 

                if vcat == 1; % married couple adjustment;
                    mu = consadj * c ^ (calc.alpha - 1); % marginal utility at the current period;
                else 
                    mu = c ^ (calc.alpha - 1);
                end;

                tu = (1 - f) * tuv(k, 1) + f * tuv(k + 1, 1) + c * mu / calc.alpha;

                if tu > tumax; % total utility is higher than the total utility from the previous solution;
                    cmax  = c; % consumption;
                    mumax = mu; % marginal utility;
                    tumax = tu; % total utility;
                end;
                
            end;

            c  = cmax; % optimal consumption at the current period;
            mu = mumax; % marginal utility at the current period;
            tu = tumax; % total utility at the current period;
            
        end;

        margu(acat, 1) = mu; % marginal utility next period (when calculating backward at age - 1);
        totu(acat, 1) = tu; % total utility next period (when calculating backward at age - 1);
        
        if vcat == 1; % both spouses alive;
            saving(acat, 1) = y - c; % save saving (coule be negative) if both spouses alive;
            totutil(acat, 1) = tu; % save total utility if both spouses alive;
        end;
        
    end;
    
else % high time preference; no assets; individual consumes all resources each period;
    
    for acat = 1 : gv.nacatlim; % initial asset;
        c = y + calc.acats(acat, 1); % this type of individuals consumes everything each period;

        if vcat == 1; % married couple adjustment;
            mu = consadj * c ^ (calc.alpha - 1); % marginal utility at the current period;
        else 
            mu = c ^ (calc.alpha - 1);
        end;

        tu = c * mu / calc.alpha; % total utility at the current period; 

        margu(acat, 1) = mu; % marginal utility next period (when calculating backward at age - 1);
        totu(acat, 1) = tu; % total utility next period (when calculating backward at age - 1);

        if vcat == 1; % both spouses alive;
            saving(acat, 1) = y - c; % save saving (coule be negative) if both spouses alive;
            totutil(acat, 1) = tu; % save total utility if both spouses alive;
        end;
    end;
    
end;
           
