% ----------------------------------------------------------------------- %
% % rhocalc function                                                    % %
% % caluclate rho consistent with initial wealth and retirement         % %
% ----------------------------------------------------------------------- %

function [calc, arglist] = rhocalc(gv, data, calc)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% calculate approximated retirement age %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% initialization; 
lastftage = 0; % observed last age working in career job;
firstprage = 99; % observed first age working in part time work;
firstretage = 99; % observed first age fully retired;

% calculate approximiated retirement age (retage);
% it is defined as the average age between age quitting career job and age first entering part time work or fully retired;
% data.retv records retirement status at ages 50 - 69; 
% 1 means fully retired, 3 means partial retired (part time work), 5 means full time work;
for age = 50 : 69;
    if data.retv(age - 50 + 1, 1) == 5; % working full time;
        if firstprage == 99 && firstretage == 99;
            lastftage = age; % observed last age working in career job;
        end;
        firstretage = 99; % refresh full retirement age;
    elseif data.retv(age - 50 + 1, 1) == 3; % partially retired (working part time);
        if firstprage == 99;
            firstprage = age; % observed partial retirement age;
        end;
        firstretage = 99; % refresh full retirement age;
    elseif data.retv(age - 50 + 1,1) == 1; % retired;
        if firstretage == 99;
            firstretage = age; % full retirement age;
        end;
    end;
end;
  
if lastftage > 0; % observed last age working in career job;
    
    if firstprage < 99; % observed partial retirement age;
        retage = floor(((lastftage + 1) + firstprage) / 2);
    elseif firstprage == 99;
        if firstretage < 99; % observed full retirement age;
            retage = floor(((lastftage + 1) + firstretage) / 2);
        elseif firstretage == 99;
            retage = lastftage + 1;
        end;
    end;

elseif lastftage == 0; % setup boundary: quit career job before 50;
    
    if firstprage < 99; % observed partial retirement age;
        retage = firstprage;
    elseif firstprage == 99;
        if firstretage < 99; % observed full retirement age;
            retage = firstretage;
        elseif firstretage == 99;
            retage = 62;
        end;
    end;

end;

% setup boundary;
if retage < 50;
    retage = 50;
elseif retage > 70;
    retage = 70;
end;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% if low assets, assign a high rho, else, find rho %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% find the highest five years of income;
wagevec = zeros(5,1);
for age = max(25, retage - 10) : retage - 1;
    earnings = data.ftwage(age - 25 + 1, 1);
    for row = 5 : -1 : 1;
        if earnings > wagevec(row, 1);
            earntemp = wagevec(row, 1);
            wagevec(row, 1) = earnings;
            earnings = earntemp;
        end;
    end;
end;

% if asset in 1992 less than 25% of income of the third highest year, and;
% total pension wealath at retirement age less than income of the third highest year;
% assign a big rho = 3.0;
% time discount rate = 1 / ( 1 + 3.0) = 0.25;
if data.assetwealth <= 0.25 * wagevec(3, 1) && data.plumpsum(retage - 50 + 1, 1) < wagevec(3, 1) 
    
    rho = 3.0;
    
else
    
    % find rho;
    % calculate conditional income for three survival status;
    calc = incomecalc(retage, gv, data, calc);
    
    % assign an initial rho value;
    % to be adjusted in later calculations;
    rho2 = 0.2; 
    
    % calculate conditional optimal consumption streams;
    calc = conscalc(rho2, gv, calc); 

    % calculate difference between observed and simulated 1992 wealth;
    wealthdiff2 = wealthcalc(gv, data, calc); 

    % if wealth difference is bigger than 0, current rho is too big;
    % if wealth difference is smaller than 0, current rho is too small;
    if wealthdiff2 >= 0; % true rho is smaller than the initial rho2;
        
        rho1 = 0; % assign a lower bound for rho;
        wealthdiff1 = -1; % initialize wealth difference associated with lower bound rho;
        
    elseif wealthdiff2 < 0; % true rho is bigger than the initial rho2;
        
        % loop three times to find upper bound for rho;
        % after three times, upper bound of rho becomes 0.2 * 2 ^ 3 = 1.6;
        % after finding a upper bound (wealthdiff2 >= 0), jump out of loop;
        for iter = 1 : 3; % iteration;
            if wealthdiff2 < 0; % true rho is bigger than current rho;
                rho1 = rho2; % replace lower bound of rho as current rho;
                wealthdiff1 = wealthdiff2; % replace wealth difference associated with lower bound rho;
                rho2 = 2 * rho1; % double current rho and redo calculations;
                calc = conscalc(rho2, gv, calc); % calculate conditional optimal consumption streams;
                wealthdiff2 = wealthcalc(gv, data, calc); % calculate difference between observed and simulated 1992 wealth;
            end;
        end;
        
    end;
    
    % after finding lower and upper bound of rho;
    % keep narrowing down rho range to match observed and simulated 1992 wealth;
    if rho1 == 0 || wealthdiff2 >= 0; % true rho is between rho1 and rho2;
        for iter = 1 : 20; % iterate 20 times to find the most accurate rho;
            rho = (rho1 + rho2) / 2; % experiement rho;
            calc = conscalc(rho, gv, calc); % calculate conditional optimal consumption streams;
            wealthdiff = wealthcalc(gv, data, calc); % calculate difference between observed and simulated 1992 wealth;
            if wealthdiff >= 0; % true rho is smaller than current rho;
                rho2 = rho; % replace upper bound of rho as current rho;
                wealthdiff2 = wealthdiff; % replace wealth difference associated with upper bound rho;
            elseif wealthdiff < 0; % true rho is bigger than current rho;
                rho1 = rho; % replace lower bound of rho as current rho;
                wealthdiff1 = wealthdiff; % replace wealth difference associated with lower bound rho;
            end;
        end;
        
        % interpolation;
        rho = (rho1 * wealthdiff2 - rho2 * wealthdiff1) / (wealthdiff2 - wealthdiff1);
    else
        
        % after iterating 3 times, if wealthdiff still smaller than 0, rho is bigger than 1.6; 
        % assign it as 1.6;
        rho = rho2; 
        
    end;
    
end;


arglist.rvec(calc.obs, 1) = rho;
















